#include <avr/io.h>
#include <avr/interrupt.h>

volatile	int counter;
volatile	char rising;

ISR(INT0_vect)
{
	if (rising == 1)
	{
		TCCR1B = (1 << CS11); /* clk / 8 start the timer */
		EICRA = 2<<ISC00;
		rising = 0;
	}
	else
	{
		TCCR1B &= 0xf8; /* stop the timer */
		counter = TCNT1/116;
		TCNT1 = 0;
		EICRA = 3<<ISC00;
		rising = 1;
	}
}
void delay_us(unsigned char time_us)
{
	register unsigned char i;
	for(i=0;i<time_us;i++) //4 cycle
	{
		asm volatile("PUSH R0"); //2 cycle
		asm volatile("POP R0"); //2 cycle
		asm volatile("PUSH R0"); //2 cycle
		asm volatile("POP R0"); //2 cycle
		asm volatile("PUSH R0"); //2 cycle
		asm volatile("POP R0"); //2 cycle
		// Sum = 16 cycle=1 us for 16MHz
	}
}
void delay_ms(unsigned int time_ms)
{
	register unsigned int i;
	for(i=0;i<time_ms;i++) //4 cycle
	{
		delay_us(250);
		delay_us(250);
		delay_us(250);
		delay_us(250);
	}
}
void display_digit(unsigned int number)
{
	unsigned char digit0, digit1, digit2, digit3;

	digit0 = number%10;
	digit1 = (number%100)/10;
	digit2 = (number%1000)/100;
	digit3 = number/1000;
	
	PORTB = 0xfb;
	PORTE = digit3;

	PORTB = 0xf7;
	PORTE = digit2;	
	
	PORTB =0xef;
	PORTE = digit1;

	PORTB =0xdf;
	PORTE = digit0;
}

int main(void)
{
	DDRB = 0xff;
	DDRE = 0xff;
	
	DDRG |= 0x10;
	
	TCCR1A = 0x00;
	TCCR1B = 0x00; 
	TCCR1C = 0x00;
	TCNT1=0;
	
	EIMSK |=1<<INT0;
	EICRA = 3<<ISC00; 
	rising = 1;
	
	sei();

	while(1){
		PORTG = 0x10;
		delay_us(15);
		PORTG = 0x00;
		
		display_digit(counter);
		delay_ms(50);
	}	
}
